From 6947ca96aafcdb1b6ae2f9433091c21de638c106 Mon Sep 17 00:00:00 2001
From: "henrique.garcia" <henrique.garcia@toradex.com>
Date: Thu, 13 Apr 2023 16:50:49 -0300
Subject: [PATCH] Enable SSCG on DRAM

---
 arch/arm/mach-imx/imx8m/clock_imx8mm.c | 44 +++++++++++++++++++++++---
 1 file changed, 40 insertions(+), 4 deletions(-)

diff --git a/arch/arm/mach-imx/imx8m/clock_imx8mm.c b/arch/arm/mach-imx/imx8m/clock_imx8mm.c
index 97143fa92c7..40503e16142 100644
--- a/arch/arm/mach-imx/imx8m/clock_imx8mm.c
+++ b/arch/arm/mach-imx/imx8m/clock_imx8mm.c
@@ -44,6 +44,7 @@ int enable_i2c_clk(unsigned char enable, unsigned i2c_num)
 
 static struct imx_int_pll_rate_table imx8mm_fracpll_tbl[] = {
 	PLL_1443X_RATE(1000000000U, 250, 3, 1, 0),
+    PLL_1443X_RATE(933000000U, 311, 4, 1, 0),
 	PLL_1443X_RATE(800000000U, 200, 3, 1, 0),
 	PLL_1443X_RATE(750000000U, 250, 2, 2, 0),
 	PLL_1443X_RATE(650000000U, 325, 3, 2, 0),
@@ -58,8 +59,8 @@ static struct imx_int_pll_rate_table imx8mm_fracpll_tbl[] = {
 static int fracpll_configure(enum pll_clocks pll, u32 freq)
 {
 	int i;
-	u32 tmp, div_val;
-	void *pll_base;
+	u32 tmp, div_val, sscg;
+	void *pll_base, *pll_dram_sscg;
 	struct imx_int_pll_rate_table *rate;
 
 	for (i = 0; i < ARRAY_SIZE(imx8mm_fracpll_tbl); i++) {
@@ -81,6 +82,7 @@ static int fracpll_configure(enum pll_clocks pll, u32 freq)
 		writel(SRC_DDR1_ENABLE_MASK, SRC_BASE_ADDR + 0x1004);
 
 		pll_base = &ana_pll->dram_pll_gnrl_ctl;
+        pll_dram_sscg = &ana_pll->dram_pll_sscg_ctl;
 		break;
 	case ANATOP_VIDEO_PLL:
 		pll_base = &ana_pll->video_pll1_gnrl_ctl;
@@ -97,6 +99,32 @@ static int fracpll_configure(enum pll_clocks pll, u32 freq)
 	tmp &= ~RST_MASK;
 	writel(tmp, pll_base);
 
+#ifdef LGE_1866MHZ
+	if(pll == ANATOP_DRAM_PLL){
+		/* SSCG enable */
+		sscg = readl(pll_dram_sscg);
+		sscg |= 1 << 31; /* SSCG_EN = 1 */
+		sscg |= 4 << 12; /* PLL_MFREQ_CTL = 4 MF = 46.875KHz */
+		sscg |= 50 << 4; /* PLL_MRAT_CTL = 50 MR = 1% */
+		sscg |= 1; /* SEL_PF = 1 */
+		writel(sscg, pll_dram_sscg);
+	}
+#else
+	if(pll == ANATOP_DRAM_PLL){
+		/* SSCG enable */
+		printf("Freq : %d \n", freq);
+		sscg = readl(pll_dram_sscg);
+		printf("SSCG ORG : %x\n", sscg);
+		sscg |= 1 << 31; /* SSCG_EN = 1 */
+		sscg |= 4 << 12; /* PLL_MFREQ_CTL = 4 MF = 62.5KHz */
+		sscg |= 60 << 4; /* PLL_MRAT_CTL = 60 MR = 1.5% */
+		sscg |= 1; /* SEL_PF = 1 */
+		printf("SSCG NOW : %x\n", sscg);
+		writel(sscg, pll_dram_sscg);
+	}
+#endif 
+
+
 	div_val = (rate->mdiv << MDIV_SHIFT) | (rate->pdiv << PDIV_SHIFT) |
 		(rate->sdiv << SDIV_SHIFT);
 	writel(div_val, pll_base + 4);
@@ -107,11 +135,19 @@ static int fracpll_configure(enum pll_clocks pll, u32 freq)
 	/* Disable RST */
 	tmp |= RST_MASK;
 	writel(tmp, pll_base);
-
+#ifdef SSCG_DEBUG
+	if(pll == ANATOP_DRAM_PLL){
+		printf("complete pll\n");
+	}
+#endif
 	/* Wait Lock*/
 	while (!(readl(pll_base) & LOCK_STATUS))
 		;
-
+#ifdef SSCG_DEBUG
+	if(pll == ANATOP_DRAM_PLL){
+		printf("pll lock\n");
+	}
+#endif
 	/* Bypass */
 	tmp &= ~BYPASS_MASK;
 	writel(tmp, pll_base);
-- 
2.34.1

